@@ -61,6 +61,7 @@ data Opts = Help
           | Language String
           | Quiet
           | Cross
+          | Ansi
             deriving Eq
 
 
@@ -83,6 +84,7 @@ opts = [Option "?" ["help"] (NoArg Help) "Display help message"
        ,Option ""  ["cpp-define"] (ReqArg Define "name[=value]") "CPP #define"
        ,Option ""  ["cpp-include"] (ReqArg Include "dir") "CPP include path"
        ,Option ""  ["cpp-simple"] (NoArg SimpleCpp) "Use a simple CPP (strip # lines)"
+       ,Option ""  ["cpp-ansi"] (NoArg Ansi) "Use CPP in -ansi mode"
        ]
 
 
@@ -115,7 +117,7 @@ getCmd args = do
     let languages = getExtensions [x | Language x <- opt]
 
     let cpphs = defaultCpphsOptions
-            {boolopts=defaultBoolOptions{hashline=False}
+            {boolopts=defaultBoolOptions{hashline=False, ansi=Ansi `elem` opt}
             ,includes = [x | Include x <- opt]
             ,defines = [(a,drop 1 b) | Define x <- opt, let (a,b) = break (== '=') x]
             }
