@@ -20,7 +20,9 @@
 
 <TEST>
 f a = \x -> x + x -- f a x = x + x
+f a = \a -> a + a -- f _ a = a + a
 f a = \x -> x + x where _ = test
+f = \x -> x + x -- f x = x + x
 fun x y z = f x y z -- fun = f
 fun x y z = f x x y z -- fun x = f x x
 fun x y z = f g z -- fun x y = f g
@@ -36,18 +38,20 @@ f = foo (\x y -> x + y) -- (+)
 f = foo (\x -> x * y) -- (* y)
 f = foo (\x -> x # y)
 f = foo (\x -> \y -> x x y y) -- \x y -> x x y y
-f = foo (\x -> \x -> foo x x)
+f = foo (\x -> \x -> foo x x) -- \_ x -> foo x x
+f = foo (\(x:xs) -> \x -> foo x x) -- \(_:xs) x -> foo x x
+f = foo (\x -> \y -> \z -> x x y y z z) -- \x y z -> x x y y z z
 x ! y = fromJust $ lookup x y
 f = foo (\i -> writeIdea (getClass i) i)
 f = bar (flip Foo.bar x) -- (`Foo.bar` x)
 f = a b (\x -> c x d)  -- (`c` d)
-yes = \x -> a x -- a
-yes = \x y -> op y x -- flip op
-f = \y -> nub $ reverse y -- nub . reverse
-f = \z -> foo $ bar $ baz z -- foo . bar . baz
-f = \z -> foo $ bar x $ baz z -- foo . bar x . baz
-f = \z -> foo $ z $ baz z
-f = \x -> bar map (filter x) -- bar map . filter
+yes = \x -> a x where -- a
+yes = \x y -> op y x where -- flip op
+f = \y -> nub $ reverse y where -- nub . reverse
+f = \z -> foo $ bar $ baz z where -- foo . bar . baz
+f = \z -> foo $ bar x $ baz z where -- foo . bar x . baz
+f = \z -> foo $ z $ baz z where
+f = \x -> bar map (filter x) where -- bar map . filter
 f = bar &+& \x -> f (g x)
 foo = [\column -> set column [treeViewColumnTitle := printf "%s (match %d)" name (length candidnates)]]
 foo = [\x -> x]
@@ -67,8 +71,8 @@ lambdaHint _ _ x = concatMap (uncurry lambdaExp) (universeParentBi x) ++ concatM
 
 
 lambdaDecl :: Decl_ -> [Idea]
-lambdaDecl o@(FunBind _ [Match _ name pats (UnGuardedRhs _ bod) Nothing])
-    | Lambda _ vs y <- bod = [err "Redundant lambda" o $ reform (pats++vs) y]
+lambdaDecl (toFunBind -> o@(FunBind _ [Match _ name pats (UnGuardedRhs _ bod) Nothing]))
+    | isLambda $ fromParen bod = [err "Redundant lambda" o $ uncurry reform $ fromLambda $ Lambda an pats bod]
     | (pats2,bod2) <- etaReduce pats bod, length pats2 < length pats = [err "Eta reduce" o $ reform pats2 bod2]
         where reform p b = FunBind an [Match an name p (UnGuardedRhs an b) Nothing]
 lambdaDecl _ = []
@@ -88,6 +92,14 @@ lambdaExp p o@(Paren _ (App _ (App _ (view -> Var_ "flip") (Var _ x)) y)) | allo
     [warn "Use section" o $ RightSection an (QVarOp an x) y]
 lambdaExp p o@Lambda{} | maybe True (not . isInfixApp) p, res <- niceLambda [] o, not $ isLambda res =
     [warn "Avoid lambda" o res]
-lambdaExp p o@(Lambda _ ps1 (fromParen -> Lambda _ ps2 bod)) | pvars ps1 `disjoint` pvars ps2 =
-    [warn "Collapse lambdas" o $ Lambda an (ps1++ps2) bod]
+lambdaExp p o@(Lambda _ _ x) | isLambda (fromParen x) && maybe True (not . isLambda) p =
+    [warn "Collapse lambdas" o $ uncurry (Lambda an) $ fromLambda o]
 lambdaExp _ _ = []
+
+
+-- replace any repeated pattern variable with _
+fromLambda :: Exp_ -> ([Pat_], Exp_)
+fromLambda (Lambda _ ps1 (fromLambda . fromParen -> (ps2,x))) = (transformBi (f $ pvars ps2) ps1 ++ ps2, x)
+    where f bad x@PVar{} | prettyPrint x `elem` bad = PWildCard an
+          f bad x = x
+fromLambda x = ([], x)
